@@ -21,12 +21,10 @@
 #include <config.h>
 #include "system.h"
 
-#include <configmake.h>
 #include <filename.h> /* IS_PATH_WITH_DIR */
 #include <get-errno.h>
 #include <path-join.h>
 #include <quotearg.h>
-#include <relocatable.h> /* relocate2 */
 #include <spawn-pipe.h>
 #include <timevar.h>
 #include <wait-process.h>
@@ -46,9 +44,6 @@
 
 static struct obstack format_obstack;
 
-/* Memory allocated by relocate2, to free.  */
-static char *relocate_buffer = NULL;
-
 
 /*-------------------------------------------------------------------.
 | Create a function NAME which associates to the muscle NAME the     |
@@ -138,9 +133,8 @@ string_output (FILE *out, char const *string)
 }
 
 
-/*----------------------------.
-| Prepare the symbols names.  |
-`----------------------------*/
+/* Generate the b4_<MUSCLE_NAME> (e.g., b4_tname) table with the
+   symbol names (aka tags). */
 
 static void
 prepare_symbol_names (char const *muscle_name)
@@ -380,7 +374,7 @@ user_actions_output (FILE *out)
                  rules[r].is_predicate ? "b4_predicate_case" : "b4_case",
                  r + 1, rules[r].action_location.start.line);
         string_output (out, rules[r].action_location.start.file);
-        fprintf (out, ")\n[    %s]])\n\n", rules[r].action);
+        fprintf (out, ")dnl\n[    %s]])\n\n", rules[r].action);
       }
   fputs ("])\n\n", out);
 }
@@ -439,13 +433,13 @@ prepare_symbol_definitions (void)
       key = obstack_finish0 (&format_obstack);
 
       /* Whether the symbol has an identifier.  */
-      const char *value = symbol_id_get (sym);
+      const char *id = symbol_id_get (sym);
       SET_KEY ("has_id");
-      MUSCLE_INSERT_INT (key, !!value);
+      MUSCLE_INSERT_INT (key, !!id);
 
       /* Its identifier.  */
       SET_KEY ("id");
-      MUSCLE_INSERT_STRING (key, value ? value : "");
+      MUSCLE_INSERT_STRING (key, id ? id : "");
 
       /* Its tag.  Typically for documentation purpose.  */
       SET_KEY ("tag");
@@ -484,6 +478,9 @@ prepare_symbol_definitions (void)
               SET_KEY2 (pname, "line");
               MUSCLE_INSERT_INT (key, p->location.start.line);
 
+              SET_KEY2 (pname, "loc");
+              muscle_location_grow (key, p->location);
+
               SET_KEY (pname);
               MUSCLE_INSERT_STRING_RAW (key, p->code);
             }
@@ -685,7 +682,7 @@ prepare (void)
 #define DEFINE(Name) MUSCLE_INSERT_STRING (#Name, Name ? Name : "")
   DEFINE (dir_prefix);
   DEFINE (parser_file_name);
-  DEFINE (spec_defines_file);
+  DEFINE (spec_header_file);
   DEFINE (spec_file_prefix);
   DEFINE (spec_graph_file);
   DEFINE (spec_name_prefix);
@@ -737,17 +734,4 @@ output (void)
     unlink_generated_sources ();
 
   obstack_free (&format_obstack, NULL);
-  free (relocate_buffer);
-}
-
-char const *
-pkgdatadir (void)
-{
-  if (relocate_buffer)
-    return relocate_buffer;
-  else
-    {
-      char const *cp = getenv ("BISON_PKGDATADIR");
-      return cp ? cp : relocate2 (PKGDATADIR, &relocate_buffer);
-    }
 }
