@@ -26,6 +26,7 @@
 #include "complain.h"
 #include "conflicts.h"
 #include "files.h"
+#include "fixits.h"
 #include "getargs.h"
 #include "gram.h"
 #include "muscle-tab.h"
@@ -226,11 +227,9 @@ grammar_current_rule_begin (symbol *lhs, location loc,
   ++nrules;
   previous_rule_end = grammar_end;
 
-  symbol_list *p = grammar_symbol_append (lhs, loc);
+  current_rule = grammar_symbol_append (lhs, loc);
   if (lhs_name)
-    assign_named_ref (p, named_ref_copy (lhs_name));
-
-  current_rule = grammar_end;
+    assign_named_ref (current_rule, named_ref_copy (lhs_name));
 
   /* Mark the rule's lhs as a nonterminal if not already so.  */
   if (lhs->content->class == unknown_sym)
@@ -293,7 +292,7 @@ grammar_rule_check_and_complete (symbol_list *r)
       if (first_rhs)
         {
           char const *lhs_type = r->content.sym->content->type_name;
-          const char *rhs_type =
+          char const *rhs_type =
             first_rhs->content->type_name ? first_rhs->content->type_name : "";
           if (!UNIQSTR_EQ (lhs_type, rhs_type))
             complain (&r->location, Wother,
@@ -346,14 +345,22 @@ grammar_rule_check_and_complete (symbol_list *r)
   /* Check that %empty => empty rule.  */
   if (r->percent_empty_loc.start.file
       && r->next && r->next->content.sym)
-    complain (&r->percent_empty_loc, complaint,
-              _("%%empty on non-empty rule"));
+    {
+      complain (&r->percent_empty_loc, complaint,
+                _("%%empty on non-empty rule"));
+      fixits_register (&r->percent_empty_loc, "");
+    }
 
   /* Check that empty rule => %empty.  */
   if (!(r->next && r->next->content.sym)
       && !r->midrule_parent_rule
-      && !r->percent_empty_loc.start.file)
-    complain (&r->location, Wempty_rule, _("empty rule without %%empty"));
+      && !r->percent_empty_loc.start.file
+      && warning_is_enabled (Wempty_rule))
+    {
+      complain (&r->location, Wempty_rule,
+                _("empty rule without %%empty"));
+      fixits_register (&r->location, " %empty ");
+    }
 
   /* See comments in grammar_current_rule_prec_set for how POSIX
      mandates this complaint.  It's only for identifiers, so skip
@@ -619,7 +626,7 @@ packgram (void)
       /* If the midrule's $$ is set or its $n is used, remove the '$' from the
          symbol name so that it's a user-defined symbol so that the default
          %destructor and %printer apply.  */
-      if (lhs->midrule_parent_rule
+      if (lhs->midrule_parent_rule /* i.e., symbol_is_dummy (lhs->content.sym).  */
           && (lhs->action_props.is_value_used
               || (symbol_list_n_get (lhs->midrule_parent_rule,
                                      lhs->midrule_parent_rhs_index)
@@ -629,8 +636,8 @@ packgram (void)
       /* Don't check the generated rule 0.  It has no action, so some rhs
          symbols may appear unused, but the parsing algorithm ensures that
          %destructor's are invoked appropriately.  */
-      if (p != grammar)
-        grammar_rule_check_and_complete (p);
+      if (lhs != grammar)
+        grammar_rule_check_and_complete (lhs);
 
       rules[ruleno].user_number = ruleno;
       rules[ruleno].number = ruleno;
@@ -645,13 +652,13 @@ packgram (void)
       rules[ruleno].action = lhs->action_props.code;
       rules[ruleno].action_location = lhs->action_props.location;
       rules[ruleno].is_predicate = lhs->action_props.is_predicate;
-      rules[ruleno].expected_sr_conflicts = p->expected_sr_conflicts;
-      rules[ruleno].expected_rr_conflicts = p->expected_rr_conflicts;
+      rules[ruleno].expected_sr_conflicts = lhs->expected_sr_conflicts;
+      rules[ruleno].expected_rr_conflicts = lhs->expected_rr_conflicts;
 
       /* Traverse the rhs.  */
       {
         size_t rule_length = 0;
-        for (p = p->next; p->content.sym; p = p->next)
+        for (p = lhs->next; p->content.sym; p = p->next)
           {
             ++rule_length;
 
@@ -754,11 +761,12 @@ prepare_percent_define_front_end_variables (void)
 
   /* Check %define front-end variables.  */
   {
-    static char const * const values[] = {
-      "lr.type", "lalr", "ielr", "canonical-lr", NULL,
-      "lr.default-reduction", "most", "consistent", "accepting", NULL,
-      NULL
-    };
+    static char const * const values[] =
+      {
+       "lr.type", "lr""(0)", "lalr", "ielr", "canonical-lr", NULL,
+       "lr.default-reduction", "most", "consistent", "accepting", NULL,
+       NULL
+      };
     muscle_percent_define_check_values (values);
   }
 }
@@ -829,8 +837,7 @@ check_and_convert_grammar (void)
   aver (nsyms <= SYMBOL_NUMBER_MAXIMUM);
   aver (nsyms == ntokens + nvars);
 
-  /* Assign the symbols their symbol numbers.  Write #defines for the
-     token symbols into FDEFINES if requested.  */
+  /* Assign the symbols their symbol numbers.  */
   symbols_pack ();
 
   /* Scan rule actions after invoking symbol_check_alias_consistency (in
